@@ -24,40 +24,61 @@
 #endif
 
 #include <stdlib.h>
+#ifdef HAVE_PTHREAD_H
+#include <pthread.h>
+#endif
+#ifdef _WIN32
+#define NOGDI
+#include <windows.h>
+#endif
 
-#if defined(HAVE_LIBUSB) && !defined(__APPLE__)
+#if defined(HAVE_HIDAPI)
+#define USE_HIDAPI
 #define USBHID
+#elif defined(HAVE_LIBUSB) && !defined(__APPLE__)
+#define USE_LIBUSB
+#define USBHID
+#endif
+
+#if defined(USE_LIBUSB)
 #ifdef _WIN32
 #define NOGDI
 #endif
 #include <libusb-1.0/libusb.h>
-#elif defined(HAVE_HIDAPI)
-#define USBHID
+#elif defined(USE_HIDAPI)
 #include <hidapi/hidapi.h>
 #endif
 
 #include "usbhid.h"
 #include "common-private.h"
 #include "context-private.h"
+#include "platform.h"
+
+#ifdef _WIN32
+typedef LONG dc_mutex_t;
+#define DC_MUTEX_INIT 0
+#else
+typedef pthread_mutex_t dc_mutex_t;
+#define DC_MUTEX_INIT PTHREAD_MUTEX_INITIALIZER
+#endif
 
 struct dc_usbhid_t {
 	/* Library context. */
 	dc_context_t *context;
 	/* Internal state. */
-#if defined(HAVE_LIBUSB) && !defined(__APPLE__)
-	libusb_context *ctx;
+#if defined(USE_LIBUSB)
 	libusb_device_handle *handle;
 	int interface;
 	unsigned char endpoint_in;
 	unsigned char endpoint_out;
 	unsigned int timeout;
-#elif defined(HAVE_HIDAPI)
+#elif defined(USE_HIDAPI)
 	hid_device *handle;
 	int timeout;
 #endif
 };
 
-#if defined(HAVE_LIBUSB) && !defined(__APPLE__)
+#if defined(USE_LIBUSB)
 static dc_status_t
 syserror(int errcode)
 {
@@ -80,13 +101,169 @@ syserror(int errcode)
 }
 #endif
 
+static dc_status_t
+usbhid_packet_close(dc_custom_io_t *io)
+{
+	dc_usbhid_t *usbhid = (dc_usbhid_t *)io->userdata;
+	return dc_usbhid_close(usbhid);
+}
+
+static dc_status_t
+usbhid_packet_read(dc_custom_io_t *io, void* data, size_t size, size_t *actual)
+{
+	dc_usbhid_t *usbhid = (dc_usbhid_t *)io->userdata;
+	return dc_usbhid_read(usbhid, data, size, actual);
+}
+
+/*
+ * FIXME! The USB HID "report type" is a disaster, and there's confusion
+ * between libusb and HIDAPI. The Scubapro G2 seems to need an explicit
+ * report type of 0 for HIDAPI, but not for libusb.
+ *
+ * See commit d251b37 ("Add a zero report ID to the commands") for the
+ * Scubapro G2 - but that doesn't actually work with the BLE case, so
+ * I really suspect that we need to do something _here_ in the packet
+ * IO layer, and have the USBHID registration set the report type to
+ * use (ie an extra new argument to dc_usbhid_custom_io() to set the
+ * report type, or something).
+ *
+ * The Suunto EON Steel just uses 0x3f and does that in the caller.
+ */
+static dc_status_t
+usbhid_packet_write(dc_custom_io_t *io, const void* data, size_t size, size_t *actual)
+{
+	dc_usbhid_t *usbhid = (dc_usbhid_t *)io->userdata;
+	return dc_usbhid_write(usbhid, data, size, actual);
+}
+
+dc_status_t
+dc_usbhid_custom_io (dc_context_t *context, unsigned int vid, unsigned int pid)
+{
+	dc_usbhid_t *usbhid;
+	dc_status_t status;
+
+	static dc_custom_io_t custom = {
+		.packet_size = 64,
+		.packet_close = usbhid_packet_close,
+		.packet_read  = usbhid_packet_read,
+		.packet_write = usbhid_packet_write,
+	};
+
+	status = dc_usbhid_open(&usbhid, context, vid, pid);
+	if (status != DC_STATUS_SUCCESS)
+		return status;
+
+	custom.userdata = (void *)usbhid;
+	dc_context_set_custom_io(context, &custom, NULL);
+
+	dc_usbhid_set_timeout(usbhid, 10);
+
+	/* Get rid of any pending stale input first */
+	/* NOTE! This will cause an annoying warning from dc_usbhid_read() */
+	for (;;) {
+		size_t transferred = 0;
+		unsigned char buf[64];
+
+		dc_status_t rc = dc_usbhid_read(usbhid, buf, sizeof(buf), &transferred);
+		if (rc != DC_STATUS_SUCCESS)
+			break;
+		if (!transferred)
+			break;
+	}
+
+	dc_usbhid_set_timeout(usbhid, 5000);
+
+	return DC_STATUS_SUCCESS;
+}
+
+#ifdef USBHID
+static dc_mutex_t g_usbhid_mutex = DC_MUTEX_INIT;
+static size_t g_usbhid_refcount = 0;
+#ifdef USE_LIBUSB
+static libusb_context *g_usbhid_ctx = NULL;
+#endif
+
+static void
+dc_mutex_lock (dc_mutex_t *mutex)
+{
+#ifdef _WIN32
+	while (InterlockedCompareExchange (mutex, 1, 0) == 1) {
+		SleepEx (0, TRUE);
+	}
+#else
+	pthread_mutex_lock (mutex);
+#endif
+}
+
+static void
+dc_mutex_unlock (dc_mutex_t *mutex)
+{
+#ifdef _WIN32
+	InterlockedExchange (mutex, 0);
+#else
+	pthread_mutex_unlock (mutex);
+#endif
+}
+
+static dc_status_t
+dc_usbhid_init (dc_context_t *context)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+
+	dc_mutex_lock (&g_usbhid_mutex);
+
+	if (g_usbhid_refcount == 0) {
+#if defined(USE_LIBUSB)
+		int rc = libusb_init (&g_usbhid_ctx);
+		if (rc != LIBUSB_SUCCESS) {
+			ERROR (context, "Failed to initialize usb support (%s).",
+				libusb_error_name (rc));
+			status = syserror (rc);
+			goto error;
+		}
+#elif defined(USE_HIDAPI)
+		int rc = hid_init();
+		if (rc < 0) {
+			ERROR (context, "Failed to initialize usb support.");
+			status = DC_STATUS_IO;
+			goto error;
+		}
+#endif
+	}
+
+	g_usbhid_refcount++;
+
+error:
+	dc_mutex_unlock (&g_usbhid_mutex);
+	return status;
+}
+
+static dc_status_t
+dc_usbhid_exit (void)
+{
+	dc_mutex_lock (&g_usbhid_mutex);
+
+	if (--g_usbhid_refcount == 0) {
+#if defined(USE_LIBUSB)
+		libusb_exit (g_usbhid_ctx);
+		g_usbhid_ctx = NULL;
+#elif defined(USE_HIDAPI)
+		hid_exit ();
+#endif
+	}
+
+	dc_mutex_unlock (&g_usbhid_mutex);
+
+	return DC_STATUS_SUCCESS;
+}
+#endif
+
 dc_status_t
 dc_usbhid_open (dc_usbhid_t **out, dc_context_t *context, unsigned int vid, unsigned int pid)
 {
 #ifdef USBHID
 	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_usbhid_t *usbhid = NULL;
-	int rc = 0;
 
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
@@ -103,21 +280,19 @@ dc_usbhid_open (dc_usbhid_t **out, dc_context_t *context, unsigned int vid, unsi
 	// Library context.
 	usbhid->context = context;
 
-#if defined(HAVE_LIBUSB) && !defined(__APPLE__)
-	struct libusb_device **devices = NULL;
-	struct libusb_config_descriptor *config = NULL;
-
-	// Initialize the libusb library.
-	rc = libusb_init (&usbhid->ctx);
-	if (rc != LIBUSB_SUCCESS) {
-		ERROR (context, "Failed to initialize usb support (%s).",
-			libusb_error_name (rc));
-		status = syserror (rc);
+	// Initialize the usb library.
+	status = dc_usbhid_init (context);
+	if (status != DC_STATUS_SUCCESS) {
 		goto error_free;
 	}
 
+#if defined(USE_LIBUSB)
+	struct libusb_device **devices = NULL;
+	struct libusb_config_descriptor *config = NULL;
+	int rc = 0;
+
 	// Enumerate the USB devices.
-	ssize_t ndevices = libusb_get_device_list (usbhid->ctx, &devices);
+	ssize_t ndevices = libusb_get_device_list (g_usbhid_ctx, &devices);
 	if (ndevices < 0) {
 		ERROR (context, "Failed to enumerate the usb devices (%s).",
 			libusb_error_name (ndevices));
@@ -235,22 +410,13 @@ dc_usbhid_open (dc_usbhid_t **out, dc_context_t *context, unsigned int vid, unsi
 	libusb_free_config_descriptor (config);
 	libusb_free_device_list (devices, 1);
 
-#elif defined(HAVE_HIDAPI)
-
-	// Initialize the hidapi library.
-	rc = hid_init();
-	if (rc < 0) {
-		ERROR (context, "Failed to initialize usb support.");
-		status = DC_STATUS_IO;
-		goto error_free;
-	}
-
+#elif defined(USE_HIDAPI)
 	// Open the USB device.
 	usbhid->handle = hid_open (vid, pid, NULL);
 	if (usbhid->handle == NULL) {
 		ERROR (context, "Failed to open the usb device.");
 		status = DC_STATUS_IO;
-		goto error_hid_exit;
+		goto error_usb_exit;
 	}
 
 	usbhid->timeout = -1;
@@ -260,19 +426,16 @@ dc_usbhid_open (dc_usbhid_t **out, dc_context_t *context, unsigned int vid, unsi
 
 	return DC_STATUS_SUCCESS;
 
-#if defined(HAVE_LIBUSB) && !defined(__APPLE__)
+#if defined(USE_LIBUSB)
 error_usb_close:
 	libusb_close (usbhid->handle);
 error_usb_free_config:
 	libusb_free_config_descriptor (config);
 error_usb_free_list:
 	libusb_free_device_list (devices, 1);
-error_usb_exit:
-	libusb_exit (usbhid->ctx);
-#elif defined(HAVE_HIDAPI)
-error_hid_exit:
-	hid_exit ();
 #endif
+error_usb_exit:
+	dc_usbhid_exit ();
 error_free:
 	free (usbhid);
 	return status;
@@ -290,14 +453,13 @@ dc_usbhid_close (dc_usbhid_t *usbhid)
 	if (usbhid == NULL)
 		return DC_STATUS_SUCCESS;
 
-#if defined(HAVE_LIBUSB) && !defined(__APPLE__)
+#if defined(USE_LIBUSB)
 	libusb_release_interface (usbhid->handle, usbhid->interface);
 	libusb_close (usbhid->handle);
-	libusb_exit (usbhid->ctx);
-#elif defined(HAVE_HIDAPI)
+#elif defined(USE_HIDAPI)
 	hid_close(usbhid->handle);
-	hid_exit();
 #endif
+	dc_usbhid_exit();
 	free (usbhid);
 
 	return status;
@@ -315,7 +477,7 @@ dc_usbhid_set_timeout (dc_usbhid_t *usbhid, int timeout)
 
 	INFO (usbhid->context, "Timeout: value=%i", timeout);
 
-#if defined(HAVE_LIBUSB) && !defined(__APPLE__)
+#if defined(USE_LIBUSB)
 	if (timeout < 0) {
 		usbhid->timeout = 0;
 	} else if (timeout == 0) {
@@ -323,7 +485,7 @@ dc_usbhid_set_timeout (dc_usbhid_t *usbhid, int timeout)
 	} else {
 		usbhid->timeout = timeout;
 	}
-#elif defined(HAVE_HIDAPI)
+#elif defined(USE_HIDAPI)
 	if (timeout < 0) {
 		usbhid->timeout = -1;
 	} else {
@@ -349,7 +511,7 @@ dc_usbhid_read (dc_usbhid_t *usbhid, void *data, size_t size, size_t *actual)
 		goto out_invalidargs;
 	}
 
-#if defined(HAVE_LIBUSB) && !defined(__APPLE__)
+#if defined(USE_LIBUSB)
 	int rc = libusb_interrupt_transfer (usbhid->handle, usbhid->endpoint_in, data, size, &nbytes, usbhid->timeout);
 	if (rc != LIBUSB_SUCCESS) {
 		ERROR (usbhid->context, "Usb read interrupt transfer failed (%s).",
@@ -357,11 +519,12 @@ dc_usbhid_read (dc_usbhid_t *usbhid, void *data, size_t size, size_t *actual)
 		status = syserror (rc);
 		goto out;
 	}
-#elif defined(HAVE_HIDAPI)
+#elif defined(USE_HIDAPI)
 	nbytes = hid_read_timeout(usbhid->handle, data, size, usbhid->timeout);
 	if (nbytes < 0) {
 		ERROR (usbhid->context, "Usb read interrupt transfer failed.");
 		status = DC_STATUS_IO;
+		nbytes = 0;
 		goto out;
 	}
 #endif
@@ -391,24 +554,50 @@ dc_usbhid_write (dc_usbhid_t *usbhid, const void *data, size_t size, size_t *act
 		goto out_invalidargs;
 	}
 
-#if defined(HAVE_LIBUSB) && !defined(__APPLE__)
-	int rc = libusb_interrupt_transfer (usbhid->handle, usbhid->endpoint_out, (void *) data, size, &nbytes, 0);
+	if (size == 0) {
+		goto out;
+	}
+
+#if defined(USE_LIBUSB)
+	const unsigned char *buffer = (const unsigned char *) data;
+	size_t length = size;
+
+	// Skip a report id of zero.
+	unsigned char report = buffer[0];
+	if (report == 0) {
+		buffer++;
+		length--;
+	}
+
+	int rc = libusb_interrupt_transfer (usbhid->handle, usbhid->endpoint_out, (void *) buffer, length, &nbytes, 0);
 	if (rc != LIBUSB_SUCCESS) {
 		ERROR (usbhid->context, "Usb write interrupt transfer failed (%s).",
 			libusb_error_name (rc));
 		status = syserror (rc);
 		goto out;
 	}
-#elif defined(HAVE_HIDAPI)
+
+	if (report == 0) {
+		nbytes++;
+	}
+#elif defined(USE_HIDAPI)
 	nbytes = hid_write(usbhid->handle, data, size);
 	if (nbytes < 0) {
 		ERROR (usbhid->context, "Usb write interrupt transfer failed.");
 		status = DC_STATUS_IO;
+		nbytes = 0;
 		goto out;
 	}
 #endif
 
 out:
+#ifdef _WIN32
+	if (nbytes > size) {
+		WARNING (usbhid->context, "Number of bytes exceeds the buffer size (" DC_PRINTF_SIZE " > " DC_PRINTF_SIZE ")!", nbytes, size);
+		nbytes = size;
+	}
+#endif
+
 	HEXDUMP (usbhid->context, DC_LOGLEVEL_INFO, "Write", (unsigned char *) data, nbytes);
 
 out_invalidargs:
